from swarms_cloud.check_model_list import (
    create_error_response,
)
from swarms_cloud.utils.calculate_pricing import calculate_pricing, count_tokens
from swarms_cloud.loggers.log_api_request_to_supabase import (
    ModelAPILogEntry,
    log_to_supabase,
)
from swarms_cloud.schema.openai_protocol import (  # noqa: E501
    ChatCompletionRequest,
    ChatCompletionRequestQos,
    ChatCompletionResponse,
    ChatCompletionResponseChoice,
    ChatCompletionResponseStreamChoice,
    ChatCompletionStreamResponse,
    ChatMessage,
    CompletionRequest,
    CompletionRequestQos,
    CompletionResponse,
    CompletionResponseChoice,
    CompletionResponseStreamChoice,
    CompletionStreamResponse,
    DeltaMessage,
    EmbeddingsRequest,
    EncodeRequest,
    EncodeResponse,
    ErrorResponse,
    GenerateRequest,
    GenerateRequestQos,
    GenerateResponse,
    GenerationConfig,
    ModelCard,
    ModelList,
    ModelPermission,
    UsageInfo,
)
from swarms_cloud.schema.openai_spec import (
    InputOpenAISpec,
    OpenAIAPIWrapper,
    OutputOpenAISpec,
)
from swarms_cloud.utils.api_key_generator import generate_api_key
from swarms_cloud.utils.rate_limiter import rate_limiter
from swarms_cloud.models import *

__all__ = [
    "create_error_response",
    "calculate_pricing",
    "count_tokens",
    "ModelAPILogEntry",
    "log_to_supabase",
    "ChatCompletionRequest",
    "ChatCompletionRequestQos",
    "ChatCompletionResponse",
    "ChatCompletionResponseChoice",
    "ChatCompletionResponseStreamChoice",
    "ChatCompletionStreamResponse",
    "ChatMessage",
    "CompletionRequest",
    "CompletionRequestQos",
    "CompletionResponse",
    "CompletionResponseChoice",
    "CompletionResponseStreamChoice",
    "CompletionStreamResponse",
    "DeltaMessage",
    "EmbeddingsRequest",
    "EncodeRequest",
    "EncodeResponse",
    "ErrorResponse",
    "GenerateRequest",
    "GenerateRequestQos",
    "GenerateResponse",
    "GenerationConfig",
    "ModelCard",
    "ModelList",
    "ModelPermission",
    "UsageInfo",
    "InputOpenAISpec",
    "OpenAIAPIWrapper",
    "OutputOpenAISpec",
    "generate_api_key",
    "rate_limiter",
]



def get_class_attributes(classes):
    class_attributes = {}
    for cls in classes:
        if isinstance(cls, type) and hasattr(cls, "__name__"):
            class_name = cls.__name__
            class_attributes[class_name] = {
                "agent_name": getattr(cls, "agent_name", None),
                "model_name": getattr(cls, "model_name", None),
                "swarm_name": getattr(cls, "swarm_name", None),
                "swarm_id": getattr(cls, "swarm_id", None),
            }
    return class_attributes



from dataclasses import dataclass
import time
from typing import Any, Dict, List, Literal, Optional, Union

import shortuuid
from pydantic import BaseModel, Field


class ErrorResponse(BaseModel):
    """Error responses."""

    object: str = "error"
    message: str
    code: int


class ModelPermission(BaseModel):
    """Model permissions."""

    id: str = Field(default_factory=lambda: f"modelperm-{shortuuid.random()}")
    object: str = "model_permission"
    created: int = Field(default_factory=lambda: int(time.time()))
    allow_create_engine: bool = False
    allow_sampling: bool = True
    allow_logprobs: bool = True
    allow_search_indices: bool = True
    allow_view: bool = True
    allow_fine_tuning: bool = False
    organization: str = "*"
    group: Optional[str] = None
    is_blocking: bool = False


class ModelCard(BaseModel):
    """Model cards."""

    id: str
    object: str = "model"
    created: int = Field(default_factory=lambda: int(time.time()))
    owned_by: str = "swarms"
    root: Optional[str] = None
    parent: Optional[str] = None
    permission: List[ModelPermission] = []


class ModelList(BaseModel):
    """Model list consists of model cards."""

    object: str = "list"
    data: List[ModelCard] = []


class UsageInfo(BaseModel):
    """Usage information."""

    prompt_tokens: int = 0
    total_tokens: int = 0
    completion_tokens: Optional[int] = 0


class ChatCompletionRequestQos(BaseModel):
    """Chat completion request."""

    model: str
    messages: Union[str, List[Dict[str, str]]]
    temperature: Optional[float] = 0.7
    top_p: Optional[float] = 1.0
    n: Optional[int] = 1
    max_tokens: Optional[int] = 512
    stop: Optional[bool] = False
    stream: Optional[bool] = False
    presence_penalty: Optional[float] = 0.0
    frequency_penalty: Optional[float] = 0.0
    user: Optional[str] = None
    user_id: Optional[str] = None
    # additional argument of swarms
    repetition_penalty: Optional[float] = 1.0
    session_id: Optional[int] = -1
    ignore_eos: Optional[bool] = False


class ChatCompletionRequest(BaseModel):
    """Chat completion request."""

    model: str
    # yapf: disable
    messages: Union[str, List[Dict[str, str]]] = Field(examples=[[{'role': 'user', 'content': 'hi'}]])  # noqa
    temperature: Optional[float] = 0.7
    top_p: Optional[float] = 1.0
    n: Optional[int] = 1
    max_tokens: Optional[int] = 512
    stop: Optional[Union[str, List[str]]] = Field(default=None, examples=[None])  # noqa
    # yapf: enable
    stream: Optional[bool] = False
    presence_penalty: Optional[float] = 0.0
    frequency_penalty: Optional[float] = 0.0
    user: Optional[str] = None
    # additional argument of swarms
    repetition_penalty: Optional[float] = 1.0
    session_id: Optional[int] = -1
    ignore_eos: Optional[bool] = False


class ChatMessage(BaseModel):
    """Chat messages."""

    role: str
    content: str


class ChatCompletionResponseChoice(BaseModel):
    """Chat completion response choices."""

    index: int
    message: ChatMessage
    finish_reason: Optional[Literal["stop", "length"]] = None


class ChatCompletionResponse(BaseModel):
    """Chat completion response."""

    id: str = Field(default_factory=lambda: f"chatcmpl-{shortuuid.random()}")
    object: str = "chat.completion"
    created: int = Field(default_factory=lambda: int(time.time()))
    model: str
    choices: List[ChatCompletionResponseChoice]
    usage: UsageInfo


class DeltaMessage(BaseModel):
    """Delta messages."""

    role: Optional[str] = None
    content: Optional[str] = None


class ChatCompletionResponseStreamChoice(BaseModel):
    """Chat completion response stream choice."""

    index: int
    delta: DeltaMessage
    finish_reason: Optional[Literal["stop", "length"]] = None


class ChatCompletionStreamResponse(BaseModel):
    """Chat completion stream response."""

    id: str = Field(default_factory=lambda: f"chatcmpl-{shortuuid.random()}")
    object: str = "chat.completion.chunk"
    created: int = Field(default_factory=lambda: int(time.time()))
    model: str
    choices: List[ChatCompletionResponseStreamChoice]


class CompletionRequest(BaseModel):
    """Completion request."""

    model: str
    prompt: Union[str, List[Any]]
    suffix: Optional[str] = None
    temperature: Optional[float] = 0.7
    n: Optional[int] = 1
    max_tokens: Optional[int] = 16
    stop: Optional[Union[str, List[str]]] = Field(default=None, examples=[None])
    stream: Optional[bool] = False
    top_p: Optional[float] = 1.0
    logprobs: Optional[int] = None
    echo: Optional[bool] = False
    presence_penalty: Optional[float] = 0.0
    frequency_penalty: Optional[float] = 0.0
    user: Optional[str] = None
    # additional argument of swarms
    repetition_penalty: Optional[float] = 1.0
    session_id: Optional[int] = -1
    ignore_eos: Optional[bool] = False
    top_k: Optional[int] = 40  # for opencompass


class CompletionRequestQos(BaseModel):
    """Completion request."""

    model: str
    prompt: Union[str, List[Any]]
    suffix: Optional[str] = None
    temperature: Optional[float] = 0.7
    n: Optional[int] = 1
    max_tokens: Optional[int] = 16
    stop: Optional[Union[str, List[str]]] = None
    stream: Optional[bool] = False
    top_p: Optional[float] = 1.0
    logprobs: Optional[int] = None
    echo: Optional[bool] = False
    presence_penalty: Optional[float] = 0.0
    frequency_penalty: Optional[float] = 0.0
    user: Optional[str] = None
    # additional argument of swarms
    top_k: int = 40
    repetition_penalty: Optional[float] = 1.0
    session_id: Optional[int] = -1
    ignore_eos: Optional[bool] = False
    user_id: Optional[str] = None


class CompletionResponseChoice(BaseModel):
    """Completion response choices."""

    index: int
    text: str
    logprobs: Optional[int] = None
    finish_reason: Optional[Literal["stop", "length"]] = None


class CompletionResponse(BaseModel):
    """Completion response."""

    id: str = Field(default_factory=lambda: f"cmpl-{shortuuid.random()}")
    object: str = "text_completion"
    created: int = Field(default_factory=lambda: int(time.time()))
    model: str
    choices: List[CompletionResponseChoice]
    usage: UsageInfo


class CompletionResponseStreamChoice(BaseModel):
    """Completion response stream choice."""

    index: int
    text: str
    logprobs: Optional[float] = None
    finish_reason: Optional[Literal["stop", "length"]] = None


class CompletionStreamResponse(BaseModel):
    """Completion stream response."""

    id: str = Field(default_factory=lambda: f"cmpl-{shortuuid.random()}")
    object: str = "text_completion"
    created: int = Field(default_factory=lambda: int(time.time()))
    model: str
    choices: List[CompletionResponseStreamChoice]


class EmbeddingsRequest(BaseModel):
    """Embedding request."""

    model: str = None
    input: Union[str, List[str]]
    user: Optional[str] = None


class EmbeddingsResponse(BaseModel):
    """Embedding response."""

    object: str = "list"
    data: List[Dict[str, Any]]
    model: str
    usage: UsageInfo


class EncodeRequest(BaseModel):
    """Encode request."""

    input: Union[str, List[str]]
    do_preprocess: Optional[bool] = False
    add_bos: Optional[bool] = True


class EncodeResponse(BaseModel):
    """Encode response."""

    input_ids: Union[List[int], List[List[int]]]
    length: Union[int, List[int]]


class GenerateRequest(BaseModel):
    """Generate request."""

    prompt: Union[str, List[Dict[str, str]]]
    session_id: int = -1
    interactive_mode: bool = False
    stream: bool = False
    stop: Optional[Union[str, List[str]]] = Field(default=None, examples=[None])
    request_output_len: int = 512
    top_p: float = 0.8
    top_k: int = 40
    temperature: float = 0.8
    repetition_penalty: float = 1.0
    ignore_eos: bool = False
    cancel: Optional[bool] = False  # cancel a responding request


class GenerateRequestQos(BaseModel):
    """Generate request."""

    prompt: Union[str, List[Dict[str, str]]]
    session_id: int = -1
    interactive_mode: bool = False
    stream: bool = False
    stop: bool = False
    request_output_len: int = 512
    top_p: float = 0.8
    top_k: int = 40
    temperature: float = 0.8
    repetition_penalty: float = 1.0
    ignore_eos: bool = False
    user_id: Optional[str] = None


class GenerateResponse(BaseModel):
    """Generate response."""

    text: str
    tokens: int
    finish_reason: Optional[Literal["stop", "length"]] = None


@dataclass
class GenerationConfig:
    """generation parameters used by inference engines.

    Args:
        n (int): Define how many chat completion choices to generate for each
            input message
        max_new_tokens (int): The maximum number of tokens that can be
            generated in the chat completion
        top_p (float): An alternative to sampling with temperature, called
            nucleus sampling, where the model considers the results of the
            tokens with top_p probability mass
        top_k (int): An alternative to sampling with temperature, where
            the model considers the top_k tokens with the highest probability
        temperature (float): Sampling temperature
        repetition_penalty (float): Penalty to prevent the model from
            generating repeated words or phrases. A value larger than
            1 discourages repetition
        ignore_eos (bool): Indicator to ignore the eos_token_id or not
        random_seed (int): Seed used when sampling a token
        stop_words (List[str]): Words that stop generating further tokens
        bad_words (List[str]): Words that the engine will never generate
        min_new_tokens (int): The minimum numbers of tokens to generate,
            ignoring the number of tokens in the prompt.
    """

    n: int = 1
    max_new_tokens: int = 512
    top_p: float = 1.0
    top_k: int = 1
    temperature: float = 0.8
    repetition_penalty: float = 1.0
    ignore_eos: bool = False
    random_seed: int = None
    stop_words: List[str] = None
    bad_words: List[str] = None
    min_new_tokens: int = None



from pydantic import BaseModel, Field


class TextToVideoRequest(BaseModel):
    model_name: str = Field(
        ...,
        example="default-text-to-video-model",
        description="Name of the text-to-video model to be used",
    )
    task: str = Field(
        ...,
        example="A girl smiling",
        description="Text input that will be converted to video",
    )
    resolution: str = Field(
        default="1080p", example="1080p", description="Resolution of the output video"
    )
    length: int = Field(
        default=60, example=60, description="Desired length of the video in seconds"
    )
    style: str = Field(
        default="realistic",
        example="realistic",
        description="Stylistic parameters or themes",
    )
    n: int = Field(default=1, example=1, description="Number of videos to generate")
    output_type: str = Field(
        default="gif",
        example="gif",
        description="Type of output to be generated, e.g. video, gif, etc.",
    )
    output_path: str = Field(
        None,
        example="animate.gif",
        description="Path to save the output video",
    )


class ErrorResponse(BaseModel):
    code: str = Field(..., example="400", description="Error code in case of failure")
    message: str = Field(
        ..., example="Invalid input text", description="Detailed error message"
    )


class TextToVideoResponse(BaseModel):
    status: str = Field(..., example="success", description="Status of the response")
    request_details: TextToVideoRequest = Field(
        ..., description="Details of the initial request"
    )
    output_path: str = Field(
        None,
        example="animate.gif",
        description="URL where the generated video can be accessed",
    )
    # error: ErrorResponse = Field(
    #     None, description="Error details if the status is 'error'"
    # )



import uuid
from pydantic import BaseModel
from typing import Optional, Sequence, Dict, List

swarm_id = uuid.uuid4()


class AutoSwarmSchemaResponse(BaseModel):
    """
    Represents the schema for an auto swarm.

    Attributes:
        id (str): The ID of the swarm.
        api_key (Optional[str]): The API key for the swarm.
        swarm_name (Optional[str]): The name of the swarm.
        num_of_agents (Optional[int]): The number of agents in the swarm.
        messages (Optional[Dict[str, str]]): The messages for the swarm.
        num_loops (Optional[int]): The number of loops for the swarm.
        streaming (Optional[bool]): Indicates if the swarm is streaming.
        tasks (Optional[Sequence[str]]): The tasks for the swarm.
        max_tokens (Optional[int]): The maximum number of tokens for the swarm.
        documents (Optional[Sequence[str]]): The documents for the swarm.
        response_format (Optional[str]): The response format for the swarm.
        stopping_token (Optional[List[str]]): The stopping tokens for the swarm.
        number_of_choices (Optional[int]): The number of choices for the swarm.
    """

    id: str = str(swarm_id)
    api_key: Optional[str] = None
    swarm_name: Optional[str] = None
    num_of_agents: Optional[int] = None
    messages: Optional[Dict[str, str]] = None
    num_loops: Optional[int] = 1
    streaming: Optional[bool] = False
    tasks: Optional[Sequence[str]] = None
    max_tokens: Optional[int] = 32096
    documents: Optional[Sequence[str]] = None
    response_format: Optional[str] = None
    stopping_token: Optional[List[str]] = []
    n: Optional[int] = 1


class AutoSwarmResponse(BaseModel):
    """
    Represents the response for an auto swarm.

    Attributes:
        id (str): The ID of the auto swarm.
        swarm_name (Optional[str]): The name of the auto swarm (optional).
        num_of_agents (Optional[int]): The number of agents in the auto swarm (optional).
        messages (Optional[Dict[str, str]]): Additional messages related to the auto swarm (optional).
        num_loops (Optional[int]): The number of loops to run the auto swarm (optional, default: 1).
        streaming (Optional[bool]): Indicates if the auto swarm should be streamed (optional, default: False).
        tasks (Optional[Sequence[str]]): The tasks to be performed by the auto swarm (optional).
        max_tokens (Optional[int]): The maximum number of tokens to generate for each task (optional).
        response_format (Optional[str]): The format of the response (optional).
        stopping_token (Optional[List[str]]): The stopping token(s) for each task (optional).
        number_of_choices (Optional[int]): The number of choices to generate for each task (optional, default: 1).
    """

    id: str
    swarm_name: Optional[str] = None
    num_of_agents: Optional[int] = None
    messages: Optional[Dict[str, str]] = None
    num_loops: Optional[int] = 1
    streaming: Optional[bool] = False
    tasks: Optional[Sequence[str]] = None
    max_tokens: Optional[int] = None
    response_format: Optional[str] = None
    stopping_token: Optional[List[str]] = None
    number_of_choices: Optional[int] = 1



import torch
from transformers import PaliGemmaForConditionalGeneration, AutoProcessor
from PIL import Image
import requests
from transformers import BitsAndBytesConfig
from swarms import BaseMultiModalModel


class PaliGemma(BaseMultiModalModel):
    """
    PaliGemma is a class that represents a model for conditional generation using the PaliGemma model.

    Args:
        model_id (str): The identifier of the PaliGemma model to be used. Default is "google/paligemma-3b-mix-224".
        max_new_tokens (int): The maximum number of new tokens to be generated. Default is 20.
        skip_special_tokens (bool): Whether to skip special tokens during decoding. Default is True.
        *args: Variable length argument list.
        **kwargs: Arbitrary keyword arguments.

    Attributes:
        model_id (str): The identifier of the PaliGemma model.
        model (PaliGemmaForConditionalGeneration): The PaliGemma model for conditional generation.
        processor (AutoProcessor): The processor for the PaliGemma model.
        max_new_tokens (int): The maximum number of new tokens to be generated.
        skip_special_tokens (bool): Whether to skip special tokens during decoding.

    Methods:
        run: Runs the PaliGemma model for conditional generation.

    """

    def __init__(
        self,
        model_id: str = "google/paligemma-3b-mix-224",
        max_new_tokens: int = 1000,
        skip_special_tokens: bool = True,
        *args,
        **kwargs
    ):
        bnb_config = BitsAndBytesConfig(
            load_in_4bit=True,
            bnb_4bit_quant_type="nf4",
            bnb_4bit_compute_dtype=torch.bfloat16,
        )

        self.model_id = model_id
        self.model = PaliGemmaForConditionalGeneration.from_pretrained(
            model_id,
            quantization_config=bnb_config,
            device_map={"": 0},
            *args,
            **kwargs
        )
        self.processor = AutoProcessor.from_pretrained(model_id)
        self.max_new_tokens = max_new_tokens
        self.skip_special_tokens = skip_special_tokens

    def run(self, task: str = None, image_url: str = None, *args, **kwargs):
        """
        Runs the PaliGemma model for conditional generation.

        Args:
            task (str): The task or prompt for conditional generation.
            image_url (str): The URL of the image to be used as input.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Returns:
            str: The generated output text.

        """
        raw_image = Image.open(requests.get(image_url, stream=True).raw)
        inputs = self.processor(task, raw_image, return_tensors="pt")
        output = self.model.generate(
            **inputs, max_new_tokens=self.max_new_tokens, **kwargs
        )
        return self.processor.decode(
            output[0], skip_special_tokens=self.skip_special_tokens
        )[len(task) :]



import os
from pydantic import BaseModel
from typing import Optional, Dict
from uuid import UUID


# from swarms_cloud.auth_with_swarms_cloud import supabase_client_init


class ModelAPILogEntry(BaseModel):
    id: Optional[UUID] = None
    created_at: Optional[str] = None
    user_id: Optional[UUID] = None
    api_key_id: Optional[UUID] = None
    model_id: Optional[UUID] = None
    input_tokens: Optional[int] = None
    output_tokens: Optional[int] = None
    all_cost: Optional[float] = None
    input_cost: Optional[float] = None
    output_cost: Optional[float] = None
    messages: Optional[Dict] = None
    status: Optional[int] = None
    temperature: Optional[float] = None
    top_p: Optional[float] = None
    echo: Optional[bool] = None
    max_tokens: Optional[int] = None
    stream: Optional[bool] = None
    repetition_penalty: Optional[float] = None


async def log_to_supabase(
    table_name: str = "swarms_cloud_api_key_activities",
    entry: ModelAPILogEntry = None,
    # supabase: Client = supabase_client_init,
):
    """
    Logs an entry to Supabase.

    Args:
        table_name (str, optional): The name of the table to insert the entry into. Defaults to "swarms_cloud_api_key_activities".
        entry (LogEntry, optional): The log entry to be inserted. Defaults to None.
        supabase (Client, optional): The Supabase client instance. Defaults to None.

    Returns:
        dict: The response from the Supabase insert operation, or an error message if an exception occurs.
    """
    from supabase import create_client

    # Supabase client
    supabase = create_client(
        supabase_url=os.getenv("SUPABASE_URL"),
        supabase_key=os.getenv("SUPABASE_KEY"),
    )
    try:
        response = supabase.table(table_name).insert(entry.model_dump()).execute()
        return response
    except Exception as error:
        print(f"Error logging to Supabase: {error}")
        return error



from typing import List, Dict, Any
import uuid
from pydantic import BaseModel
from typing import Optional
import time

# ID
id = str(uuid.uuid4())


class InputOpenAISpec(BaseModel):
    """OpenAI Spec for the model"""

    model: Optional[str] = "gpt-3.5-turbo"
    max_new_tokens: Optional[int] = 100
    prompt: Optional[str] = ""
    stream: Optional[bool] = False
    sampling_params: Optional[Dict[str, Any]] = None
    best_of: Optional[int] = 1
    echo: Optional[bool] = False
    frequency_penalty: Optional[float] = 0.0
    logit_bias: Optional[Dict[str, Any]] = None
    logprobs: Optional[int] = None
    max_tokens: Optional[int] = None
    n: Optional[int] = 1
    presence_penalty: Optional[float] = 0.0
    seed: Optional[int] = None
    stop: Optional[str] = None
    suffix: Optional[str] = None
    temperature: Optional[float] = 0.0
    top_k: Optional[int] = 0
    top_p: Optional[float] = 1.0
    user: Optional[str] = None


class OutputOpenAISpec(BaseModel):
    """OpenAI Spec for the model"""

    id: Optional[str] = id
    created: Optional[int] = int(time.time())
    object: Optional[str] = None
    system_fingerpoint: Optional[str] = None
    model: Optional[str] = "gpt-3.5-turbo"
    max_new_tokens: Optional[int] = 100
    prompt: Optional[str] = ""
    stream: Optional[bool] = False
    sampling_params: Optional[Dict[str, Any]] = None
    best_of: Optional[int] = 1
    echo: Optional[bool] = False
    frequency_penalty: Optional[float] = 0.0
    logit_bias: Optional[Dict[str, Any]] = None
    logprobs: Optional[int] = None
    max_tokens: Optional[int] = None
    n: Optional[int] = 1
    presence_penalty: Optional[float] = 0.0
    seed: Optional[int] = None
    stop: Optional[str] = None
    suffix: Optional[str] = None
    temperature: Optional[float] = 0.0
    top_k: Optional[int] = 0
    top_p: Optional[float] = 1.0
    user: Optional[str] = None
    usage: Optional[Dict[str, Any]] = None
    completion_tokens: Optional[int] = None
    prompt_tokens: Optional[int] = None
    total_tokens: Optional[int] = None
    choices: Optional[List[Dict[str, Any]]] = None
    finish_reason: Optional[str] = None
    index: Optional[int] = None
    logprobs: Optional[Dict[str, Any]] = None
    text: Optional[str] = None
    conversation_id: Optional[str] = None
    response_id: Optional[str] = None
    timestamp: Optional[str] = None
    status: Optional[str] = None
    error: Optional[str] = None
    error_message: Optional[str] = None
    error_code: Optional[int] = None
    error_details: Optional[str] = None
    error_traceback: Optional[str] = None
    error_cause: Optional[str] = None
    error_type: Optional[str] = None
    error_context: Optional[str] = None


class OpenAIAPIWrapper:
    """
    A wrapper class for the OpenAI API.

    This class provides methods to set and get the input and output specifications for the OpenAI API.
    """

    def __init__(self):
        self.input_spec = InputOpenAISpec()
        self.output_spec = OutputOpenAISpec()

    def set_input_spec(self, **kwargs):
        """
        Set the input specification for the OpenAI API.

        Args:
            **kwargs: Keyword arguments representing the input specification attributes and their values.
        """
        for key, value in kwargs.items():
            if hasattr(self.input_spec, key):
                setattr(self.input_spec, key, value)

    def set_output_spec(self, **kwargs):
        """
        Set the output specification for the OpenAI API.

        Args:
            **kwargs: Keyword arguments representing the output specification attributes and their values.
        """
        for key, value in kwargs.items():
            if hasattr(self.output_spec, key):
                setattr(self.output_spec, key, value)

    def get_input_spec(self):
        """
        Get the input specification for the OpenAI API.

        Returns:
            str: JSON representation of the input specification.
        """
        return self.input_spec.json()

    def get_output_spec(self):
        """
        Get the output specification for the OpenAI API.

        Returns:
            str: JSON representation of the output specification.
        """
        return self.output_spec.json()



from http import HTTPStatus

from fastapi.responses import JSONResponse

from swarms_cloud.schema.openai_protocol import (  # noqa: E501
    ErrorResponse,
)


def create_error_response(status: HTTPStatus, message: str):
    """Create error response according to http status and message.

    Args:
        status (HTTPStatus): HTTP status codes and reason phrases
        message (str): error message
    """
    return JSONResponse(
        ErrorResponse(
            message=message, type="invalid_request_error", code=status.value
        ).model_dump()
    )



import secrets
import string


def generate_api_key(prefix: str = "sk-", length: int = 50, *args, **kwargs):
    """Generate a random api key

    Args:
        prefix (str, optional): Prefix. Defaults to "sk-".
        length (int, optional): length of the apikey. Defaults to 50.

    Raises:
        ValueError: _description_
        RuntimeError: _description_

    Returns:
        _type_: _description_


    Example:
    >>> from swarms_cloud.api_key_generator import generate_api_key
    >>> generate_api_key()
    """
    if length <= len(prefix):
        raise ValueError("Length must be greater than prefix length")

    try:
        # Generate a random string of letters for the api
        characters = string.ascii_letters + string.digits
        secure_key = "".join(secrets.choice(characters) for _ in range(length))
        return prefix + secure_key
    except Exception as error:
        # Handle unexpeccted errors
        raise RuntimeError(f"Error generating api key: {error}")
